/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.kylin.rest.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.kylin.rest.service.ServiceTestBase;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.security.authentication.TestingAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;

/**
 * @author xduo
 */
public class UserControllerTest extends ServiceTestBase {

    private UserController userController;

    @BeforeClass
    public static void setupResource() {
        List<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();
        User user = new User("ADMIN", "ADMIN", authorities);
        Authentication authentication = new TestingAuthenticationToken(user, "ADMIN", "ROLE_ADMIN");
        SecurityContextHolder.getContext().setAuthentication(authentication);
    }

    @Before
    public void setup() throws Exception {
        super.setUp();

        userController = new UserController();
    }

    @Test
    public void testBasics() throws IOException {
        UserDetails user = userController.authenticate();
        Assert.assertNotNull(user);
        Assert.assertTrue(user.getUsername().equals("ADMIN"));
    }
}
